@@ -1,8 +1,8 @@
 /* Online channel data structure.
  *
- * IRC Services is copyright (c) 1996-2002 Andrew Church.
+ * IRC Services is copyright (c) 1996-2004 Andrew Church.
  *     E-mail: <achurch@achurch.org>
- * Parts copyright (c) 1999-2000 Andrew Kempe and others.
+ * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
  * details.
  */
@@ -14,6 +14,7 @@
 
 struct channel_ {
     Channel *next, *prev;
+
     char name[CHANMAX];
     ChannelInfo *ci;			/* Corresponding ChannelInfo */
     time_t creation_time;		/* When channel was created */
@@ -25,18 +26,23 @@ struct channel_ {
     int32 mode;				/* CMODE_* (binary) channel modes */
     int32 limit;			/* 0 if none */
     char *key;				/* NULL if none */
+    char *link;				/* +L (Unreal, trircd) */
+    char *flood;			/* +f (Unreal, etc.) */
+    int32 joindelay;			/* +J (trircd) */
+    int32 joinrate1, joinrate2;		/* +j (Bahamut) */
 
-    int32 bancount, bansize;
     char **bans;
-#ifdef HAVE_BANEXCEPT
-    int32 exceptcount, exceptsize;
+    int32 bans_count;
     char **excepts;
-#endif
+    int32 excepts_count;
+    char **invites;
+    int32 invites_count;
 
     struct c_userlist {
 	struct c_userlist *next, *prev;
 	User *user;
 	int32 mode;			/* CUMODE_* modes (chanop, voice) */
+	int16 flags;			/* CUFLAG_* flags (below) */
     } *users;
 
     time_t server_modetime;		/* Time of last server MODE */
@@ -46,6 +52,9 @@ struct channel_ {
     int16 bouncy_modes;			/* Did we fail to set modes here? */
 };
 
+/* Set by ChanServ if it deops a user on joining a channel */
+#define CUFLAG_DEOPPED	0x0001
+
 /*************************************************************************/
 
 #endif
