@@ -1,14 +1,37 @@
 /* Compatibility routines.
  *
- * IRC Services is copyright (c) 1996-2002 Andrew Church.
+ * IRC Services is copyright (c) 1996-2004 Andrew Church.
  *     E-mail: <achurch@achurch.org>
- * Parts copyright (c) 1999-2000 Andrew Kempe and others.
+ * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
  * details.
  */
 
 #include "services.h"
 
+#if !HAVE_HSTRERROR
+# include <netdb.h>
+#endif
+
+/*************************************************************************/
+
+#if !HAVE_HSTRERROR
+
+/* hstrerror: return an error message for the given h_errno code. */
+
+const char *hstrerror(int h_errnum)
+{
+    switch (h_errnum) {
+	case HOST_NOT_FOUND: return "Host not found";
+	case TRY_AGAIN:      return "Host not found, try again";
+	case NO_RECOVERY:    return "Nameserver error";
+	case NO_DATA:        return "No data of requested type";
+	default:             return "Unknown error";
+    }
+}
+
+#endif /* !HAVE_HSTRERROR */
+
 /*************************************************************************/
 
 #if !HAVE_SNPRINTF
@@ -35,9 +58,12 @@ int vsnprintf(char *buf, size_t len, const char *fmt, va_list args)
 int snprintf(char *buf, size_t len, const char *fmt, ...)
 {
     va_list args;
+    int ret;
 
     va_start(args, fmt);
-    return vsnprintf(buf, len, fmt, args);
+    ret = vsnprintf(buf, len, fmt, args);
+    va_end(args);
+    return ret;
 }
 
 #endif /* !HAVE_SNPRINTF */
@@ -47,7 +73,9 @@ int snprintf(char *buf, size_t len, const char *fmt, ...)
 #if !HAVE_STRTOK
 
 /* glibc 2.2 (RedHat 7.0) has a broken strtok--it dies if called with a
- * NULL parameter after returning NULL once.
+ * NULL parameter after returning NULL once.  glibc and possibly other
+ * libraries also return non-NULL for strtok(NULL, "") even after
+ * returning NULL for strtok(NULL, " ").
  */
 
 char *strtok(char *str, const char *delim)
