@@ -1,21 +1,26 @@
 /* Declarations of IRC message structures, variables, and functions.
  *
- * IRC Services is copyright (c) 1996-2002 Andrew Church.
+ * IRC Services is copyright (c) 1996-2004 Andrew Church.
  *     E-mail: <achurch@achurch.org>
- * Parts copyright (c) 1999-2000 Andrew Kempe and others.
+ * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
  * details.
  */
 
 /*************************************************************************/
 
+/* Type of a message function: */
+typedef void (*MessageFunc)(char *source, int ac, char **av);
+
 typedef struct {
     const char *name;
-    void (*func)(char *source, int ac, char **av);
+    MessageFunc func;
 } Message;
 
-extern Message messages[];
-
+extern int register_messages(Message *table);
+extern int unregister_messages(Message *table);
 extern Message *find_message(const char *name);
+extern int messages_init(int ac, char **av);
+extern void messages_cleanup(void);
 
 /*************************************************************************/
