@@ -1,110 +1,18 @@
 /* Main processing code for Services.
  *
- * IRC Services is copyright (c) 1996-2002 Andrew Church.
+ * IRC Services is copyright (c) 1996-2004 Andrew Church.
  *     E-mail: <achurch@achurch.org>
- * Parts copyright (c) 1999-2000 Andrew Kempe and others.
+ * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
  * details.
  */
 
 #include "services.h"
+#include "modules.h"
 #include "messages.h"
+#include "ignore.h"
 
-/*************************************************************************/
-/*************************************************************************/
-
-/* Use ignore code? */
-int allow_ignore = 1;
-
-/* People to ignore. */
-static IgnoreData *ignore[1024];
-#define HASH(nick) (((nick)[0]&31)<<5 | ((nick)[1]&31))
-
-/*************************************************************************/
-
-/* first_ignore, next_ignore: Iterate over the ignore list. */
-
-static int iterator_char = 256;   /* return NULL initially */
-static IgnoreData *iterator_ptr = NULL;
-
-IgnoreData *first_ignore(void)
-{
-    iterator_char = -1;
-    iterator_ptr = NULL;
-    return next_ignore();
-}
-
-IgnoreData *next_ignore(void)
-{
-    if (iterator_ptr)
-	iterator_ptr = iterator_ptr->next;
-    while (!iterator_ptr && iterator_char < 256) {
-	iterator_char++;
-	if (iterator_char < 256)
-	    iterator_ptr = ignore[iterator_char];
-    }
-    return iterator_ptr;
-}
-
-/*************************************************************************/
-
-/* add_ignore: Add someone to the ignorance list for the next `delta'
- *             seconds.
- */
-
-void add_ignore(const char *nick, time_t delta)
-{
-    IgnoreData *ign;
-    char who[NICKMAX];
-    time_t now = time(NULL);
-    IgnoreData **whichlist = &ignore[HASH(nick)];
-
-    strscpy(who, nick, NICKMAX);
-    for (ign = *whichlist; ign; ign = ign->next) {
-	if (stricmp(ign->who, who) == 0)
-	    break;
-    }
-    if (ign) {
-	if (ign->time > now)
-	    ign->time += delta;
-	else
-	    ign->time = now + delta;
-    } else {
-	ign = smalloc(sizeof(*ign));
-	strscpy(ign->who, who, sizeof(ign->who));
-	ign->time = now + delta;
-	ign->next = *whichlist;
-	*whichlist = ign;
-    }
-}
-
-/*************************************************************************/
-
-/* get_ignore: Retrieve an ignorance record for a nick.  If the nick isn't
- *             being ignored, return NULL and flush the record from the
- *             in-core list if it exists (i.e. ignore timed out).
- */
-
-IgnoreData *get_ignore(const char *nick)
-{
-    IgnoreData *ign, *prev;
-    time_t now = time(NULL);
-    IgnoreData **whichlist = &ignore[HASH(nick)];
-
-    for (ign = *whichlist, prev = NULL; ign; prev = ign, ign = ign->next) {
-	if (irc_stricmp(ign->who, nick) == 0)
-	    break;
-    }
-    if (ign && ign->time <= now) {
-	if (prev)
-	    prev->next = ign->next;
-	else
-	    *whichlist = ign->next;
-	free(ign);
-	ign = NULL;
-    }
-    return ign;
-}
+static int cb_recvmsg = -1;
 
 /*************************************************************************/
 /*************************************************************************/
@@ -118,41 +26,64 @@ IgnoreData *get_ignore(const char *nick)
  *             RFC.  Destroys the buffer by side effect.
  */
 
+static char **sbargv = NULL;
+
 int split_buf(char *buf, char ***argv_ptr, int colon_special)
 {
     static int argvsize = 8;
-    static char **argv = NULL;
     int argc;
     char *s;
 
-    if (!argv)
-	argv = smalloc(sizeof(char *) * argvsize);
+    if (!sbargv)
+	sbargv = smalloc(sizeof(char *) * argvsize);
     argc = 0;
     while (*buf) {
 	if (argc == argvsize) {
 	    argvsize += 8;
-	    argv = srealloc(argv, sizeof(char *) * argvsize);
+	    sbargv = srealloc(sbargv, sizeof(char *) * argvsize);
 	}
 	if (*buf == ':' && colon_special) {
-	    argv[argc++] = buf+1;
-	    buf = "";
+	    sbargv[argc++] = buf+1;
+	    *buf = 0;
 	} else {
 	    s = strpbrk(buf, " ");
 	    if (s) {
 		*s++ = 0;
-		while (isspace(*s))
+		while (*s == ' ')
 		    s++;
 	    } else {
 		s = buf + strlen(buf);
 	    }
-	    argv[argc++] = buf;
+	    sbargv[argc++] = buf;
 	    buf = s;
 	}
     }
-    *argv_ptr = argv;
+    *argv_ptr = sbargv;
     return argc;
 }
 
+/*************************************************************************/
+/*************************************************************************/
+
+int process_init(int ac, char **av)
+{
+    cb_recvmsg = register_callback(NULL, "receive message");
+    if (cb_recvmsg < 0) {
+	log("process_init: register_callback() failed\n");
+	return 0;
+    }
+    return 1;
+}
+
+/*************************************************************************/
+
+void process_cleanup(void)
+{
+    unregister_callback(NULL, cb_recvmsg);
+    free(sbargv);
+    sbargv = NULL;
+}
+
 /*************************************************************************/
 
 /* process:  Main processing routine.  Takes the string in inbuf (global
@@ -166,7 +97,6 @@ void process()
     char *s;
     int ac;			/* Parameters for the command */
     char **av;
-    Message *m;
 
 
     /* If debugging, log the buffer. */
@@ -186,7 +116,7 @@ void process()
 	while (isspace(*++s))
 	    ;
 	strscpy(source, buf+1, sizeof(source));
-	memmove(buf, s, strlen(s)+1);
+	strmove(buf, s);
     } else {
 	*source = 0;
     }
@@ -203,13 +133,23 @@ void process()
     ac = split_buf(s, &av, 1);
 
     /* Do something with the message. */
-    m = find_message(cmd);
-    if (m) {
-	if (m->func)
-	    m->func(source, ac, av);
-    } else {
-	log("unknown message from server (%s)", inbuf);
+    if (call_callback_4(NULL, cb_recvmsg, source, cmd, ac, av) <= 0) {
+	if (!source && stricmp(cmd, "NICK") != 0) {
+	    log("Source missing for `%s' message from server (%s)",cmd,inbuf);
+	} else {
+	    Message *m = find_message(cmd);
+	    if (m) {
+		if (m->func)
+		    m->func(source, ac, av);
+	    } else {
+		log("unknown message from server (%s)", inbuf);
+	    }
+	}
     }
+
+    /* Finally, clear the first byte of `inbuf' to signal that we're
+     * finished processing. */
+    *inbuf = 0;
 }
 
 /*************************************************************************/
