@@ -2,62 +2,29 @@
  * by Andrew Kempe (TheShadow)
  *     E-mail: <theshadow@shadowfire.org>
  *
- * IRC Services is copyright (c) 1996-2002 Andrew Church.
+ * IRC Services is copyright (c) 1996-2004 Andrew Church.
  *     E-mail: <achurch@achurch.org>
- * Parts copyright (c) 1999-2000 Andrew Kempe and others.
+ * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
  * details.
  */
 
 #include "services.h"
-#include "pseudo.h"
+#include "modules.h"
+#include "hash.h"
 
 /*************************************************************************/
 
-static Server *serverlist;	/* Pointer to first server in the list */
+void add_server(Server *server);  /* to avoid "no prototype" warning */
+void del_server(Server *server);  /* same */
+DEFINE_HASH(server, Server, name)
+
+static Server *root_server;	/* Entry for root server (Services) */
 static int16 servercnt = 0;	/* Number of online servers */
 
-static Server *lastserver = NULL; /* Points to the server last returned by
-				   * findserver(). This should improve
-				   * performance during netbursts of NICKs. */
-
-/*************************************************************************/
-
-/*************************************************************************/
-/****************************** Statistics *******************************/
-/*************************************************************************/
-
-/* Return information on memory use. Assumes pointers are valid. */
-
-void get_server_stats(long *nservers, long *memuse)
-{
-    Server *server;
-    long mem;
-
-    mem = sizeof(Server) * servercnt;
-    for (server = serverlist; server; server = server->next)
-	mem += strlen(server->name)+1;
-
-    *nservers = servercnt;
-    *memuse = mem;
-}
-
-/*************************************************************************/
-
-#ifdef DEBUG_COMMANDS
-void send_server_list(User *user)
-{
-    Server *server;
-
-    notice(s_OperServ, user->nick, "Servers: %d", servercnt);
-    for (server = serverlist; server; server = server->next) {
-	notice(s_OperServ, user->nick, "%s [%s] {%s}", server->name,
-			server->hub ? server->hub->name : "no hub",
-			server->child ? server->child->name : "no child");
-
-    }
-}
-#endif /* DEBUG_COMMANDS */
+/* Callback IDs: */
+static int cb_create = -1;
+static int cb_delete = -1;
 
 /*************************************************************************/
 /**************************** Internal Functions *************************/
@@ -74,69 +41,83 @@ static Server *new_server(const char *servername)
     servercnt++;
     server = scalloc(sizeof(Server), 1);
     server->name = sstrdup(servername);
-
-    server->next = serverlist;
-    if (server->next)
-	server->next->prev = server;
-    serverlist = server;
-
+    add_server(server);
     return server;
 }
 
+
 /* Remove and free a Server structure. */
 
 static void delete_server(Server *server)
 {
-    if (debug >= 2)
-	log("debug: delete_server() called");
-
-    if (server == lastserver)
-	lastserver = NULL;
-
+    del_server(server);
     servercnt--;
     free(server->name);
-    if (server->prev)
-	server->prev->next = server->next;
-    else
-	serverlist = server->next;
-    if (server->next)
-	server->next->prev = server->prev;
     free(server);
+}
 
+/*************************************************************************/
+
+/* Remove the given server.  This takes care of recursively removing child
+ * servers, handling NOQUIT, calling the server delete callback, and
+ * actually deleting the server data.
+ */
+
+static void recursive_squit(Server *parent, const char *reason);
+
+static void squit_server(Server *server, const char *reason)
+{
+    User *user, *nextuser;
+
+    recursive_squit(server, reason);
+    if (protocol_features & PF_NOQUIT) {
+#define next snext
+#define prev sprev
+	LIST_FOREACH_SAFE (user, server->userlist, nextuser)
+	    quit_user(user, reason, 0);
+#undef next
+#undef prev
+    }
+    if (!server->fake)
+	call_callback_2(NULL, cb_delete, server, reason);
+    delete_server(server);
+}
+
+
+/* "SQUIT" all servers who are linked to us via the specified server by
+ * deleting them from the server list. The parent server is not deleted,
+ * so this must be done by the calling function.
+ */
+
+static void recursive_squit(Server *parent, const char *reason)
+{
+    Server *server, *nextserver;
+
+    server = parent->child;
     if (debug >= 2)
-	log("debug: delete_server() done");
+	log("debug: recursive_squit, parent: %s", parent->name);
+    while (server) {
+	nextserver = server->sibling;
+	if (debug >= 2)
+	    log("debug: recursive_squit, child: %s", server->name);
+	squit_server(server, reason);
+	server = nextserver;
+    }
 }
 
 /*************************************************************************/
 /**************************** External Functions *************************/
 /*************************************************************************/
 
-Server *findserver(const char *servername)
-{
-    Server *server;
-
-    if (!servername)
-	return NULL;
-
-    if (lastserver && stricmp(servername, lastserver->name) == 0)
-	return lastserver;
-
-    for (server = serverlist; server; server = server->next) {
-	if (stricmp(servername, server->name) == 0) {
-	    lastserver = server;
-	    return server;
-	}
-    }
-
-    return NULL;
-}
-
-/*************************************************************************/
-/*************************************************************************/
-
 /* Handle a server SERVER command.
  * 	source = server's hub; !*source indicates this is our hub.
  * 	av[0]  = server's name
+ * If ac < 0, the server in av[0] is assumed to be a "dummy" server that
+ * should not be passed to the callback (e.g. for JUPE).
+ *
+ * When called internally to add a server (from OperServ JUPE, etc.),
+ * callers may assume that the contents of the argument strings will not be
+ * modified.
  */
 
 void do_server(const char *source, int ac, char **av)
@@ -148,26 +129,23 @@ void do_server(const char *source, int ac, char **av)
     server->child = NULL;
     server->sibling = NULL;
 
-#ifdef STATISTICS
-    server->stats = stats_do_server(av[0], source);
-#endif
-
-    if (!*source)
-	return;
-
-    server->hub = findserver(source);
-    if (!server->hub) {
-	/* PARANOIA: This should NEVER EVER happen, but we check anyway.
-	 *
-	 * I've heard that on older ircds it is possible for "source" not to
-	 * be the new server's hub. This will cause problems. -TheShadow */
-	wallops(s_OperServ,
-		"WARNING: Could not find server \2%s\2 which is supposed to "
-		"be the hub for \2%s\2", source, av[0]);
-	log("server: could not find hub %s for %s", source, av[0]);
-	return;
+    if (*source) {
+	server->hub = get_server(source);
+	if (!server->hub) {
+	    /* PARANOIA: This should NEVER EVER happen, but we check anyway.
+	     *
+	     * I've heard that on older ircds it is possible for "source"
+	     * not to be the new server's hub. This will cause problems.
+	     * -TheShadow
+	     */
+	    wallops(ServerName,
+		    "WARNING: Could not find server \2%s\2 which is supposed "
+		    "to be the hub for \2%s\2", source, av[0]);
+	    log("server: could not find hub %s for %s", source, av[0]);
+	}
+    } else {
+	server->hub = root_server;
     }
-
     if (!server->hub->child) {
 	server->hub->child = server;
     } else {
@@ -177,37 +155,16 @@ void do_server(const char *source, int ac, char **av)
 	tmpserver->sibling = server;
     }
 
+    if (ac > 0)
+	call_callback_1(NULL, cb_create, server);
+    else
+	server->fake = 1;
+
     return;
 }
 
 /*************************************************************************/
 
-/* "SQUIT" all servers who are linked to us via the specified server by
- * deleting them from the server list. The parent server is not deleted,
- * so this must be done by the calling function.
- */
-
-void recursive_squit(Server *parent, const char *reason)
-{
-    Server *server, *nextserver;
-
-    server = parent->child;
-    if (debug >= 2)
-	log("recursive_squit, parent: %s", parent->name);
-    while (server) {
-	if (server->child)
-	    recursive_squit(server, reason);
-	if (debug >= 2)
-	    log("recursive_squit, child: %s", server->name);
-	nextserver = server->sibling;
-#ifdef STATISTICS
-	stats_do_squit(server, reason);
-#endif
-	delete_server(server);
-	server = nextserver;
-    }
-}
-
 /* Handle a server SQUIT command.
  * 	av[0] = server's name
  * 	av[1] = quit message
@@ -215,16 +172,16 @@ void recursive_squit(Server *parent, const char *reason)
 
 void do_squit(const char *source, int ac, char **av)
 {
-    Server *server, *tmpserver;
+    Server *server;
 
-    server = findserver(av[0]);
+    server = get_server(av[0]);
 
     if (server) {
-	recursive_squit(server, av[1]);
 	if (server->hub) {
 	    if (server->hub->child == server) {
 		server->hub->child = server->sibling;
 	    } else {
+		Server *tmpserver;
 		for (tmpserver = server->hub->child; tmpserver->sibling;
 				tmpserver = tmpserver->sibling) {
 		    if (tmpserver->sibling == server) {
@@ -233,20 +190,70 @@ void do_squit(const char *source, int ac, char **av)
 		    }
 		}
 	    }
-#ifdef STATISTICS
-	    stats_do_squit(server, av[1]);
-#endif
-	    delete_server(server);
 	}
+	squit_server(server, av[1]);
 
     } else {
-	wallops(s_OperServ,
+	wallops(ServerName,
 		"WARNING: Tried to quit non-existent server: \2%s", av[0]);
 	log("server: Tried to quit non-existent server: %s", av[0]);
-	/* FIXME: debug code (why do we get weird hostnames here?) */
 	log("server: Input buffer: %s", inbuf);
 	return;
     }
 }
 
 /*************************************************************************/
+/*************************************************************************/
+
+int server_init(int ac, char **av)
+{
+    Server *server;
+
+    cb_create = register_callback(NULL, "server create");
+    cb_delete = register_callback(NULL, "server delete");
+    if (cb_create < 0 || cb_delete < 0) {
+	log("server_init: register_callback() failed\n");
+	return 0;
+    }
+    server = new_server("");
+    server->fake = 1;
+    server->t_join = time(NULL);
+    server->hub = server->child = server->sibling = NULL;
+    root_server = server;
+    return 1;
+}
+
+/*************************************************************************/
+
+/* Remove all servers; this recursively takes out all users and channels,
+ * as well.
+ */
+
+void server_cleanup(void)
+{
+    uint32 pf = protocol_features;
+    protocol_features |= PF_NOQUIT;
+    squit_server(root_server, "server_cleanup");
+    protocol_features = pf;
+    unregister_callback(NULL, cb_delete);
+    unregister_callback(NULL, cb_create);
+}
+
+/*************************************************************************/
+
+/* Return information on memory use. Assumes pointers are valid. */
+
+void get_server_stats(long *nservers, long *memuse)
+{
+    Server *server;
+    long mem;
+
+    mem = sizeof(Server) * servercnt;
+    for (server = first_server(); server; server = next_server())
+	mem += strlen(server->name)+1;
+
+    *nservers = servercnt;
+    *memuse = mem;
+}
+
+/*************************************************************************/
