@@ -1,8 +1,8 @@
 /* Online server data.
  *
- * IRC Services is copyright (c) 1996-2002 Andrew Church.
+ * IRC Services is copyright (c) 1996-2004 Andrew Church.
  *     E-mail: <achurch@achurch.org>
- * Parts copyright (c) 1999-2000 Andrew Kempe and others.
+ * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
  * details.
  */
@@ -14,15 +14,17 @@
 
 struct server_ {
     Server *next, *prev;        /* Use to navigate the entire server list */
-    Server *hub;
-    Server *child, *sibling;
+    Server *hub;		/* Server's hub from our point of view */
+    Server *child, *sibling;	/* Server's children from our P.O.V. */
 
+    int fake;			/* Is this a "fake" (root/juped) server? */
     char *name;                 /* Server's name */
     time_t t_join;		/* Time server joined us. 0 == not here. */
 
-#ifdef STATISTICS
+    User *userlist;		/* List of users on server.  NOTE: this is
+				 * linked via snext/sprev, not next/prev. */
+
     ServerStats *stats;
-#endif
 };
 
 /*************************************************************************/
