@@ -1,8 +1,8 @@
 /* Time-delay routine include stuff.
  *
- * IRC Services is copyright (c) 1996-2002 Andrew Church.
+ * IRC Services is copyright (c) 1996-2004 Andrew Church.
  *     E-mail: <achurch@achurch.org>
- * Parts copyright (c) 1999-2000 Andrew Kempe and others.
+ * Parts written by Andrew Kempe and others.
  * This program is free but copyrighted software; see the file COPYING for
  * details.
  */
@@ -12,29 +12,37 @@
 
 #include <time.h>
 
+/*************************************************************************/
+
+/* Timeout type. */
 
-/* Definitions for timeouts: */
 typedef struct timeout_ Timeout;
+
 struct timeout_ {
-    Timeout *next, *prev;
+    void *data;			/* Caller data; can be anything */
     time_t settime;		/* Time timer was set (from time()) */
+    /* Remainder is PRIVATE DATA! */
+#ifdef IN_TIMEOUT_C
+    Timeout *next, *prev;
     uint32 timeout;		/* In milliseconds (time_msec()) */
-    int repeat;			/* Does this timeout repeat indefinitely? */
+    uint32 repeat;		/* Does this timeout repeat indefinitely?
+				 *    (if nonzero, new value of `timeout') */
     void (*code)(Timeout *);	/* This structure is passed to the code */
-    void *data;			/* Can be anything */
+#endif
 };
 
+/*************************************************************************/
 
 /* Check the timeout list for any pending actions. */
 extern void check_timeouts(void);
 
-/* Add a timeout to the list to be triggered in `delay' seconds.  Any
- * timeout added from within a timeout routine will not be checked during
- * that run through the timeout list.  Always succeeds.
- */
+/* Add a timeout to the list to be triggered in `delay' seconds (`delay'
+ * may be zero).  Any timeout added from within a timeout routine will not
+ * be checked during that run through the timeout list.  Always succeeds. */
 extern Timeout *add_timeout(int delay, void (*code)(Timeout *), int repeat);
 
-/* Add a timeout to the list to be triggered in `delay' milliseconds. */
+/* Add a timeout to the list to be triggered in `delay' milliseconds
+ * (`delay' may be zero). */
 extern Timeout *add_timeout_ms(uint32 delay, void (*code)(Timeout *),
 			       int repeat);
 
@@ -46,5 +54,6 @@ extern void del_timeout(Timeout *t);
 extern void send_timeout_list(User *u);
 #endif
 
+/*************************************************************************/
 
 #endif	/* TIMEOUT_H */
